import numpy as np

def get_lattice_generators(t: float) -> np.ndarray:
    omega_1 = get_omega_t(1)
    omega_t = get_omega_t(t)

    return np.array([1, omega_1, omega_t, omega_1 * omega_t])


def get_omega_t(t: float) -> complex:
    return np.exp(1j * np.arccos(1 - 1 / 2 / t))


def get_unit_length_count_in_grid(
        coeff_abs_max: int,
        generators: np.ndarray
    ) -> int:
    """
    Let `generators = [z_1,...,z_k]` be a complex vector. Then this function
    outputs the number of entries of unit length (up to numerical precision)
    in the grid
    ```
    {c_1z_1 + ... + c_kz_k : c_i in [-coeff_abs_max, ..., coeff_abs_max]}
    ```
    """
    raise NotImplementedError


def test_get_edge_count_in_grid():
    alpha = ((1 - 13 ** .5) + 1j * (2 + 52 ** .5) ** .5) / 4
    infinite = alpha ** np.arange(4)

    for coeff_abs_max, edge_count_true, generators in (
        (2, 6, np.array((1, get_omega_t(1)))),
        (2, 18, get_lattice_generators(3)), # https://arxiv.org/abs/2406.15317
        (2, 12, get_lattice_generators(5)),
        (2, 12, get_lattice_generators(13)),
        (5, 24, get_lattice_generators(13)), # https://arxiv.org/abs/2410.16172
        (2, 16, infinite),
        (4, 20, infinite),
        (8, 24, infinite),
        (16, 28, infinite) # https://arxiv.org/abs/1807.03726
    ):
        edge_count_gotten = get_unit_length_count_in_grid(coeff_abs_max, generators)
        if edge_count_gotten != edge_count_true:
            raise ValueError(f"With {coeff_abs_max = } and {generators = }, we have {edge_count_gotten = } != {edge_count_true}")

    print("All tests passed. Congratulations!")


if __name__ == "__main__":
    test_get_edge_count_in_grid()